import { faker } from '@faker-js/faker'
import { Browser, BrowserContext, Locator, Page, expect } from '@playwright/test'
import { checkA11y, configureAxe, injectAxe } from 'axe-playwright'

import { test } from './testmaster'

// sometimes requests to the app backends take a little longer (10s < response latency < 40s)
export const donationsRequestTimeout = 40_000
export const goodnewsRequestTimeout = 20_000
export const volunteeringRequestTimeout = 20_000

// see https://playwright.dev/docs/selectors#id-data-testid-data-test-id-data-test-selectors
export const byTestId = (testId: string, nth?: number) => `[data-testid='${testId}']${nth ? ` >> nth=${nth}` : ''}`

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export const delay = (time: number) =>
  test.expect(false, "don't use delays, they are hiding test issues that we should fix directly").toBeTruthy()

export const locateElementByTestId = async (page: Page, testId: string, nth?: number): Promise<Locator> => {
  return page.locator(byTestId(testId, nth))
}

export const clickFirstElement = async (locator: Locator): Promise<void> => {
  await locator.first().click()
}

export const clickElementByTestId = async (page: Page, testId: string, nth?: number): Promise<void> => {
  return clickFirstElement(await locateElementByTestId(page, testId, nth))
}

/*
 * Only use for text that is provided by the test (fixture)
 */
export const clickButtonByText = async (page: Page, buttonText: string): Promise<void> => {
  await page.click(`button:has-text("${buttonText}")`)
}

const prepareForLoginOrRegistration = async (context: BrowserContext, page: Page) => {
  await context.clearCookies()
  await page.goto('/profile')
}

export const gotoFillAndSubmitLogin = async (
  context: BrowserContext,
  page: Page,
  email: string,
  password: string
): Promise<void> => {
  await prepareForLoginOrRegistration(context, page)

  await clickElementByTestId(page, 'btn-to-login-form')

  const emailInput = await locateElementByTestId(page, 'ory-identifier-input')
  await emailInput.fill(email)

  const passwordInput = await locateElementByTestId(page, 'ory-password-input')
  await passwordInput.fill(password)

  await clickElementByTestId(page, 'ory-submit-password')
}

export const registerUserAccount = async (
  context: BrowserContext,
  page: Page,
  email: string,
  password: string
): Promise<{ id: string; fullName: string }> => {
  await registerUser(context, page, email, password)
  return await finishAccountSetup(page)
}

export const registerUser = async (
  context: BrowserContext,
  page: Page,
  email: string,
  password: string
): Promise<void> => {
  await retry(async () => {
    await prepareForLoginOrRegistration(context, page)

    await clickElementByTestId(page, 'btn-to-registration-form')
    await waitForAnimationEnd(page, byTestId('stepper-scrollview'))
    await clickElementByTestId(page, 'community-guidelines-checkbox')
    await clickElementByTestId(page, 'community-guidelines-navbtn-next-btn')
    await waitForAnimationEnd(page, byTestId('stepper-scrollview'))
    await clickElementByTestId(page, 'privacy-policy-checkbox')
    await clickElementByTestId(page, 'privacy-policy-navbtn-next-btn')
    await waitForAnimationEnd(page, byTestId('stepper-scrollview'))
  })

  const emailInput = await locateElementByTestId(page, 'ory-traits-email-input')
  const passwordInput = await locateElementByTestId(page, 'ory-password-input')

  await emailInput.fill(email)
  await passwordInput.fill(password)

  await clickElementByTestId(page, 'ory-submit-password')
}

export const finishAccountSetup = async (
  page: Page,
  {
    firstName = faker.person.firstName(),
    lastName = faker.person.lastName(),
  }: {
    firstName?: string
    lastName?: string
  } = {}
): Promise<{ id: string; fullName: string }> => {
  let id
  await page.route(
    `${process.env.BASE_URL}/api/graphql?op=AuthenticatedUser`,
    async (route) => {
      try {
        const response = await route.fetch()
        const result = await response.json()
        id = result.data.authenticatedUser.id
      } finally {
        await route.continue()
      }
    },
    // as this route is called by a lot of the tested components
    // only intercept the route once
    { times: 1 }
  )

  // go through account setup
  // provide user names
  const firstNameInput = await locateElementByTestId(page, 'first-name-input')
  await firstNameInput.fill(firstName)
  const lastNameInput = await locateElementByTestId(page, 'last-name-input')
  await lastNameInput.fill(lastName)
  // submit
  await clickElementByTestId(page, 'holi-wizard-save-btn')

  // submit pre-success step
  await page.waitForSelector(byTestId('account-setup-pre-registration-success-step-headline'))
  await clickElementByTestId(page, 'holi-wizard-save-btn')
  // skip interests step
  await page.waitForSelector(byTestId('account-setup-interests-step-headline'))
  await clickElementByTestId(page, 'holi-wizard-save-btn')
  // skip location step
  await page.waitForSelector(byTestId('account-setup-location-step-headline'))
  await clickElementByTestId(page, 'holi-wizard-save-btn')
  // skip interests step
  await page.waitForSelector(byTestId('account-setup-success-step'))
  await clickElementByTestId(page, 'holi-wizard-save-btn')

  if (!id) throw new Error("Wasn't able to intercept user id while finishing account setup flow")
  return {
    id,
    fullName: firstName + ' ' + lastName,
  }
}

export const proceedAsGuest = async (page: Page): Promise<void> => {
  await page.goto('/')
  await Promise.all([
    page.waitForSelector(byTestId('btn-to-continue-as-guest'), { state: 'detached' }),
    clickElementByTestId(page, 'btn-to-continue-as-guest'),
  ])
}

export const loginAsE2EUser = async (context: BrowserContext, page: Page): Promise<void> => {
  test
    .expect(
      !!process.env.E2E_USER_USERNAME,
      'The environment variable E2E_USER_USERNAME is not set up correctly, please set it up.'
    )
    .toBeTruthy()
  test
    .expect(
      !!process.env.E2E_USER_PASSWORD,
      'The environment variable E2E_USER_PASSWORD is not set up correctly, please set it up.'
    )
    .toBeTruthy()
  await gotoFillAndSubmitLogin(
    context,
    page,
    process.env.E2E_USER_USERNAME as string,
    process.env.E2E_USER_PASSWORD as string
  )
  // wait for profile - login succeeded
  await page.waitForSelector(byTestId('user-profile-full-name'))
}

export const registerNewUserAccount = async (
  context: BrowserContext,
  page: Page,
  email?: string,
  password?: string
): Promise<{ id: string; fullName: string }> => {
  const emailToUse = email || newUserEmail()
  const passwordToUse = password || newUserPassword
  const newAccount = await registerUserAccount(context, page, emailToUse, passwordToUse)
  // wait for home feed - registration succeeded
  await page.waitForSelector(byTestId('home-feed-insights-heading'))
  return newAccount
}

export const checkAccessibility = async (page: Page) => {
  console.debug('check accessibility for', page.url())
  await injectAxe(page)
  const axeOptions = {
    rules: {
      region: { enabled: true }, // https://dequeuniversity.com/rules/axe/4.4/region
      duplicateId: false,
    },
    detailedReport: true,
    detailedReportOptions: { html: true },
  }
  // Disabled 'color-contrast' for now to not distract from more serious issues
  await configureAxe(page, {
    rules: [
      { id: 'color-contrast', enabled: false },
      { id: 'scrollable-region-focusable', enabled: false }, // TODO: Fix HoliStepper issue with this rule
      { id: 'duplicate-id', enabled: false }, // svg icon components rendered in lists have duplicated ids for their shape masks
    ],
  })
  await checkA11y(page, undefined, axeOptions, false)
}

export const sendConnectionRequest = async (page: Page) => {
  // visit another user's profile page via url
  const userId = process.env.E2E_USER_ID
  // test won't work if userId is not defined
  expect(userId?.length).toBe(36)
  await page.goto('/profile/' + userId)
  // because we do a hard navigation we need to wait for logged in user to be fetched from BE. So, wait for 'logged-in-profile'
  await page.waitForSelector(byTestId('logged-in-profile'))
  await page.waitForSelector(byTestId('user-profile-full-name'))

  // for the newly created user, the button state should be NOT_CONNECTED
  await clickElementByTestId(page, 'user-connection-button-NOT_CONNECTED')

  // after clicked, the button state should change to REQUESTED
  await page.waitForSelector(byTestId('user-connection-button-REQUESTED'), { state: 'detached' })
}

export const randomSpaceName = () => {
  return faker.animal.type() + faker.number.int()
}

export const newUserEmail = () => {
  return `${faker.internet.userName()}${faker.string.alphanumeric(4)}@e2etest.com`
}

export const randomPassword = () => {
  return faker.internet.password()
}
export const newUserPassword = 'TheSamePassword4A11E2eUsers'

export const withRandomUserSession = async (
  browser: Browser,
  actions: (page: Page, userId: string, userFullName: string) => Promise<void>,
  email: string = newUserEmail(),
  password: string = randomPassword()
): Promise<() => Promise<void>> => {
  const context = await browser.newContext()
  const page = await context.newPage()

  const newAccount = await registerNewUserAccount(context, page, email, password)
  // goto profile to simulate old behaviour for now
  await page.goto('/profile')
  await page.waitForSelector(byTestId('user-profile-full-name'))

  await actions(page, newAccount.id, newAccount.fullName)

  return async () => {
    await deleteUser(page)
    await context.close()
  }
}

const deleteUser = async (page: Page) => {
  await retry(async () => {
    await page.goto('/profile')
    await clickElementByTestId(page, 'btn-to-settings')
    await clickElementByTestId(page, 'btn-your-account')
    await clickElementByTestId(page, 'btn-delete-account')
    await clickElementByTestIdAndWaitForToast(
      page,
      'account-deletion-confirmation-confirmation-confirm',
      'toast-SUCCESS'
    )
  })
}

export const clickElementByTestIdAndWaitForToast = async (page: Page, elementId: string, toastId: string) => {
  await Promise.all([clickElementByTestId(page, elementId), page.waitForSelector(byTestId(toastId))])
}

export const clickElementByTestIdAndWaitForToastFinish = async (page: Page, elementId: string, toastId: string) => {
  await clickElementByTestIdAndWaitForToast(page, elementId, toastId)
  await page.waitForSelector(byTestId(toastId), { state: 'detached' })
}

/*
 * This function will wait for any movement or resizing of the element(s) matching the passed in locator.
 * This will NOT WAIT for fading in and out, because it does only measure changes in the bounding box of the
 * elements.
 */
export const waitForAnimationEnd = async (page: Page, locatorString: string): Promise<void> => {
  const element = page.locator(locatorString)
  await element.waitFor()
}

/*
 * Wrap around a multi-step expectation that needs its steps retried
 * for features with unpredictable background work
 * think: re-freshing or re-clicking
 *
 * Be sure to have an expect() in the expectationFn
 * otherwise, it will not throw and just pass
 *
 * Be aware that the test itself is also retried in CI
 */
export const retry = async (expectationFn: () => Promise<void>) => {
  await expect(async () => {
    await expectationFn()
  }).toPass({
    intervals: [2_000],
    timeout: 30_000,
  })
}
